package extensions.generic;
import org.escience.XSIL.*;
import java.util.*;

public class Plot3D extends XSIL {

    public String Java3DGeometryType;
	public String CoordUnit;

    public int nvertex = 0;
    public float[] colors;
	public String[] atomList;
	public float[] atomicchargeList;
	public double[] coordxList;
    public double[] coordyList;
    public double[] coordzList;

    boolean instantiated = false;


    public void construct(){
	
	colors = null;
	atomList = null;
	atomicchargeList = null;
	coordxList = null;
	coordyList = null;
	coordzList = null;

	for(int ichild=0; ichild < getChildCount(); ichild++){
            XSIL x = getChild(ichild);
            if(x instanceof Param){
		Param p = (Param)x;
		if(p.getName().equals("Java3DGeometryType"))
		    Java3DGeometryType = p.getText();
		if(p.getName().equals("Unit"))
		    CoordUnit = p.getText();		   
	    }
            if(x instanceof Table ){
		Table t = (Table)x;
		nvertex	= t.getNRow();	
	    }
	}
    }

    // This function does the heavy lifting of reading arrays and so on
    // It is separate from the construction because it is not called unless necessary.

    public void instantiate(){

	for(int ichild=0; ichild < getChildCount(); ichild++){
            XSIL x = getChild(ichild);
        if(x instanceof Param){
		Param p = (Param)x;
		if(p.getName().equals("Java3DGeometryType"))
		    Java3DGeometryType = p.getText();

		if(p.getName().equals("Unit"))
		    CoordUnit = p.getText();		
	    }
	    
        if(x instanceof Table ){
		Table t = (Table)x;
		nvertex	= t.getNRow();	

		//System.out.println("ncol: "+t.getNColumn());

		for(int i=0; i< t.getNColumn();i++) {

		    //System.out.println("i: "+i);
		    Column c = t.getColumn(i);
		    if(c.getName().equals("Atom")) {		    	
		      	atomList = c.getPrimArray().getStringArray(0, t.getNRow());
		      	if(atomList.length < t.getNRow()) {
		             System.out.println("Java3D: Atom List size is " + 
		             atomList.length + " (expected " + t.getNRow() + ")");
                     atomList = null;
                }     
		    }
		    if(c.getName().equals("AtomicCharge")) {
		    	atomicchargeList = c.getPrimArray().getFloatArray(0, t.getNRow());
		      	if(atomicchargeList.length < t.getNRow()) {
		             System.out.println("Java3D: Atom Charge List size is " + 
		             atomicchargeList.length + " (expected " + t.getNRow() + ")");
                     atomicchargeList = null;
                }     		    	
		    }
		    if(c.getName().equals("CoordX")) {
		    	coordxList = c.getPrimArray().getDoubleArray(0, t.getNRow());
		      	if(coordxList.length < t.getNRow()) {
		             System.out.println("Java3D: Coord X size is " + 
		             coordxList.length + " (expected " + t.getNRow() + ")");
                     coordxList = null;
                }     		    	
		    }
		    if(c.getName().equals("CoordY")) {
		    	coordyList = c.getPrimArray().getDoubleArray(0, t.getNRow());
		      	if(coordyList.length < t.getNRow()) {
		             System.out.println("Java3D: Coord Y size is " + 
		             coordyList.length + " (expected " + t.getNRow() + ")");
                     coordyList = null;
                }     		    	
		    }
		    if(c.getName().equals("CoordZ")) {
		    	coordzList = c.getPrimArray().getDoubleArray(0, t.getNRow());
		      	if(coordzList.length < t.getNRow()) {
		             System.out.println("Java3D: Coord Z size is " + 
		             coordzList.length + " (expected " + t.getNRow() + ")");
                     coordzList = null;
                }     		    	
		    }
		} //for t.getNColumn
	    } //if (x instanceof Table)

        if(x instanceof Array && ((Array)x).getNdim() == 2){
		Array a = (Array)x;
		if(a.getDim(1) != 3) continue;
		if(3*nvertex != a.getNdata()) continue;
		
		if(a.getName().equals("Colors")){
                  colors = a.getPrimArray().getFloatArray(0, 3*nvertex);
                  if(colors.length < 3*nvertex) {
		             System.out.println("Java3D: Colors vector size is " + 
		             colors.length + " (expected " + 3*nvertex + ")");
                     colors = null;
		           }
		}
	    }
	}
    }

    public String getJava3DGeometryType() {
	String tmp = Java3DGeometryType.trim();
	return tmp;}

    public String getCoordUnit() {
	String tmp = CoordUnit.trim();
	return tmp;}

    public String[] getAtomList() {   	
	      if(!instantiated) instantiate();	
	      return atomList; }
	      
    public float[] getAtomicChargeList() { 
	      if(!instantiated) instantiate();
		  return atomicchargeList; }
		  
    public double[] getCoordXList() {	
	      if(!instantiated) instantiate();
	      return coordxList; }
	      
    public double[] getCoordYList() {
	      if(!instantiated) instantiate();
	      return coordyList; }
	      
    public double[] getCoordZList() {
	      if(!instantiated) instantiate();
	      return coordzList; }

    public float[] getColors() {
	      if(!instantiated) instantiate();
	      return colors; }

    public int getNvertex() {return nvertex;}  

    public String toString(){
	return "Java3DGeometry [" + nvertex + " vertices]";
    }
}
